import xml.dom.minidom
import logging
import numpy as np
logger = logging.getLogger(__name__)


def read_vtu_file(file_name):
    try:
        return xml.dom.minidom.parse(file_name)
    except Exception as e:
        logger.error(f"Error parsing {file_name}: {str(e)}")
        raise


def extract_data_array(output, name):
    data_arrays = output.getElementsByTagName("DataArray")
    data_array = next(
        (da for da in data_arrays if da.attributes['Name'].value == name), None)
    if data_array is None:
        logging.error(f"Data array '{name}' not found in VTU file.")
        return None
    floats = [float(n) for n in data_array.firstChild.data.strip().split()]
    return np.array(floats, dtype=data_array.attributes["type"].value.lower())


def calculate_L2_norm(ref, data):
    return np.linalg.norm(ref - data)


def compare_vtu_files(ref_file, result_file, arrays):
    ref_output = read_vtu_file(ref_file)
    result_output = read_vtu_file(result_file)
    differences = {}
    for array in arrays:
        ref = extract_data_array(ref_output, array)
        result = extract_data_array(result_output, array)
        if ref is not None and result is not None:
            differences[f'{array}_L2_diff'] = calculate_L2_norm(ref, result)
    return differences
