import sqlite3
import logging
from contextlib import contextmanager
import os
import cbutil.postprocessing.sqlite_helper as sh

logger = logging.getLogger(__file__)


@contextmanager
def sqlite_context(path):
    if not os.path.exists(path):
        raise ValueError(f"{path} does not exists")

    logging.info(f"trying to connect to {path}")
    connection = sqlite3.connect(path)
    connection.row_factory = sqlite3.Row
    try:
        yield connection
    finally:
        logging.info(f"closing {connection}")
        connection.close()


def get_all_table_names(connection):
    return [t[0] for t in connection.execute(sh.table_name_query()).fetchall()]


def query_complete_table(connection, table_name: str):
    return connection.execute(f"{sh.select_stmt()} {sh.from_stmt(table_name)}")


def query_join(connection, lhs, rhs, key):
    return connection.execute(f"{sh.select_stmt()} {sh.from_stmt(lhs)} {sh.join_stmt(lhs, rhs, key)}")


def tables2dict(table_iterator):
    for table in table_iterator:
        yield dict(table)


def iterate_all_tables(path):
    with sqlite_context(path) as connection:
        for table_name in get_all_table_names(connection):
            yield from tables2dict(query_complete_table(connection, table_name))


def iterate_join(path, lhs, rhs, key):
    with sqlite_context(path) as connection:
        yield from tables2dict(query_join(connection, lhs, rhs, key))


def iterate_query(connection, query):
    for result in connection.execute(query).fetchall():
        yield result


def build_iterate_query(connection, *args, **kwargs):
    query = sh.query_builder(*args, **kwargs)
    yield from iterate_query(connection, query)
