import yaml


class Reference:
    """Represents the !reference tags from Gitlab Ci yamls."""
    yaml_tag = '!reference'

    def __init__(self, values):
        self._values = values


def reference_constructor(loader: yaml.SafeLoader, node: yaml.nodes.CollectionNode) -> Reference:
    return Reference(loader.construct_sequence(node))


def reference_representer(dumper: yaml.SafeDumper, ref: Reference) -> yaml.nodes.SequenceNode:
    return dumper.represent_sequence(Reference.yaml_tag, ref._values, flow_style=True)


def loader():
    loader = yaml.SafeLoader
    loader.add_constructor(Reference.yaml_tag, reference_constructor)
    return loader


def dumper():
    dumper = yaml.SafeDumper
    dumper.add_representer(Reference, reference_representer)
    dumper.ignore_aliases = lambda *args: True
    return dumper


def load_script(raw_strings):
    if isinstance(raw_strings, str):
        raw_strings = [raw_strings]
    for raw_string in raw_strings:
        yield yaml.load(raw_string, Loader=loader())


def print_yaml_file(config: dict) -> str:
    return yaml.dump(config,
                     default_flow_style=False,
                     line_break=False,
                     width=120,
                     Dumper=dumper(),
                     sort_keys=False)
