import argparse
import logging

import dashboards.dashboard_list as boards
from dashboards.upload import upload_dashboard
from dashboards.dashboard_fe2ti import dashboard_fe2ti
from dashboard_pystencils import dashboard_pystencils_cpu, dashboard_pystencils_gpu
from dashboard_walberla import dashboard_uniformgridcpu, dashboard_uniformgridgpu, dashboard_uniformgridgpu_profile, dashboard_fslbmgravitywave, dashboard_uniformgridcpu_relativeperformance, dashboard_percolationgpu

logger = logging.getLogger(__file__)
logger.setLevel(logging.INFO)


def get_available_dashboards():
    yield from (func for func in dir(boards) if func.startswith("dashboard_"))


def strip_prefix(available):
    return available.replace("dashboard_", "")


def main():
    available_dashboards = list(get_available_dashboards())
    parser = argparse.ArgumentParser(description="List Dashboards you want to deploy")
    parser.add_argument("dashboards",
                        nargs='*',
                        help=', '.join((strip_prefix(d) for d in available_dashboards)))
    args = parser.parse_args()
    if len(args.dashboards) == 0:
        dashboards = available_dashboards
    else:
        dashboards = (f"dashboard_{db}" for db in args.dashboards)

    walberla_folder = 8
    fe2ti_folder = 40
    pystencils_foler = 41
    for board_name in dashboards:
        logger.info(f"Try to upload {board_name}")
        if "fe2ti" in board_name:
            upload_dashboard(dashboard_fe2ti(), folder=fe2ti_folder)
        elif "pystencils" in board_name:
            upload_dashboard(dashboard_pystencils_cpu(), folder=pystencils_foler)
            upload_dashboard(dashboard_pystencils_gpu(), folder=pystencils_foler)
        elif "walberla" in board_name:
            upload_dashboard(dashboard_uniformgridcpu(), folder=walberla_folder)
            upload_dashboard(dashboard_uniformgridgpu(), folder=walberla_folder)
            upload_dashboard(dashboard_uniformgridgpu_profile(), folder=walberla_folder)
            upload_dashboard(dashboard_fslbmgravitywave(), folder=walberla_folder)
            upload_dashboard(dashboard_uniformgridcpu_relativeperformance(), folder=walberla_folder)
            upload_dashboard(dashboard_percolationgpu(), folder=walberla_folder)
        else:
            board = getattr(boards, board_name)
            upload_dashboard(board(), folder=walberla_folder)


if __name__ == "__main__":
    main()
