from grafanalib.core import Template
from collections import namedtuple
from dashboards.influx_queries import show_tag_values, get_tag_values, show_field_keys
from typing import List

"""
multi: maps to the multi_value option
refresh: maps to the grafanalib.Template refresh 1= on dashboard load 2= on timer range change
"""
Filter = namedtuple(
    "Filter", ("name", "multi", "default_value", "refresh"), defaults=("", True, "", 1)
)


def get_measurement_filter(
    name, data_source, *, filter_pattern: str = None, **kwargs
) -> Template:
    query = "SHOW MEASUREMENTS"
    if filter_pattern:
        query += f" WITH MEASUREMENT =~ /{filter_pattern}/"
    return get_dashboard_variable_query(name, query, data_source, **kwargs)


def get_dashboard_variable(filter: Filter, measurment_name: str, data_source: str):
    query = show_tag_values(measurment_name, filter.name)
    kwargs = {
        "includeAll": filter.multi,
        "multi": filter.multi,
    }
    if filter.default_value:
        kwargs.update({"default": filter.default_value})
    return get_dashboard_variable_query(filter.name, query, data_source, **kwargs)


def get_time_dependend_dashboard_variable(
    filter: Filter,
    measurment_name: str,
    data_source: str,
    *,
    inner_field_key: str = "",
    where: str = "",
    group_by: List[str] = None
):
    query = get_tag_values(
        measurment_name,
        filter.name,
        inner_field_key=inner_field_key,
        where=where,
        group_by=group_by
    )
    kwargs = {
        "includeAll": filter.multi,
        "multi": filter.multi,
        "refresh": filter.refresh
    }
    if filter.default_value:
        kwargs.update({"default": filter.default_value})
    return get_dashboard_variable_query(filter.name, query, data_source, **kwargs)


def get_field_keys_dashboard_variable(filter: Filter, measurment_name: str, data_source: str):
    query = show_field_keys(measurment_name)
    kwargs = {
        "includeAll": filter.multi,
        "multi": filter.multi,
    }
    if filter.default_value:
        kwargs.update({"default": filter.default_value})
    return get_dashboard_variable_query(filter.name, query, data_source, **kwargs)


def get_dashboard_variable_query(
    name: str,
    template_query: str,
    dataSource: str,
    *,
    includeAll=True,
    multi=True,
    **kwargs,
) -> Template:
    return Template(
        name,
        template_query,
        dataSource=dataSource,
        type="query",
        includeAll=includeAll,
        multi=multi,
        **kwargs,
    )
