(use-modules
  (guix packages)
  (guix download)
  (guix build-system python)
  (guix build-system pyproject)
  (guix licenses)
  (gnu packages version-control)
  (gnu packages python-xyz)
  (gnu packages python-build)
  (gnu packages python-check)
  (gnu packages python-xyz)
  (gnu packages check)
  (gnu packages)
  )

(define-public python-pytest-html
               (package
                 (name "python-pytest-html")
                 (version "3.2.0")
                 (source (origin
                           (method url-fetch)
                           (uri (pypi-uri "pytest-html" version))
                           (sha256
                             (base32
                               "1hw7wvxiibq4wq24d2gnqaxq3pvi7s54l5yj3bskgi7z1fxz9qn4"))))
                 (build-system python-build-system)
                 (propagated-inputs (list python-py python-pytest python-pytest-metadata))
                 (native-inputs (list python-setuptools-scm))
                 (home-page "https://github.com/pytest-dev/pytest-html")
                 (synopsis "pytest plugin for generating HTML reports")
                 (description "pytest plugin for generating HTML reports")
                 (license #f)))

(define-public python-randomgen
               (package
                 (name "python-randomgen")
                 (version "1.23.1")
                 (source (origin
                           (method url-fetch)
                           (uri (pypi-uri "randomgen" version))
                           (sha256
                             (base32
                               "0z61msi94pj8yx0xxk44qf5xhmawysfn9j7h1939lqplzkl1jqw2"))))
                 (build-system python-build-system)
                 (propagated-inputs (list python-numpy python-cython))
                 (native-inputs (list python-wheel python-packaging python-cython python-pytest python-setuptools python-setuptools-scm))
                 (home-page "https://github.com/bashtage/randomgen")
                 (synopsis "Random generator supporting multiple PRNGs")
                 (description "Random generator supporting multiple PRNGs")
                 (license #f)))

(define-public python-pystencils
               (package
                 (name "python-pystencils")
                 (version "1.1.1")
                 (source (origin
                           (method url-fetch)
                           (uri (pypi-uri "pystencils" version))
                           (sha256
                             (base32
                               "02n5r1rn5pdiy037wnpx2786mbq1ngs732jkdch7mm57sx2m8yj7"))))
                 (build-system python-build-system)
                 (arguments
                   '(#:phases
                     (modify-phases %standard-phases
                                    (replace 'check
                                             (lambda _
                                               (setenv "HOME" "/tmp")
                                               (invoke "python" "conftest.py"))))))
                 (propagated-inputs (list python-appdirs
                                          python-joblib
                                          python-numpy
                                          python-sympy
                                          python-cython))
                 (native-inputs
                   (list
                     python-pytest
                     python-pytest-cov
                     python-pytest-html
                     python-pytest-xdist
                     python-ansi2html
                     python-flake8
                     python-nbformat
                     python-nbconvert
                     python-ipython
                     python-randomgen))
                 (home-page "https://i10git.cs.fau.de/pycodegen/pystencils/")
                 (synopsis "Speeding up stencil computations on CPUs and GPUs")
                 (description "Speeding up stencil computations on CPUs and GPUs")
                 (license #f)))

(define-public python-lbmpy
               (package
                 (name "python-lbmpy")
                 (version "1.1.1")
                 (source (origin
                           (method url-fetch)
                           (uri (pypi-uri "lbmpy" version))
                           (sha256
                             (base32
                               "13rwry7bijijzdllnzdx3kdzydxjvrlihp1n6aal3jf9z7yfgmrx"))))
                 (build-system python-build-system)
                 (arguments
                   '(#:phases
                     (modify-phases %standard-phases
                                    (replace 'check
                                             (lambda _
                                               (setenv "HOME" "/tmp")
                                               (invoke "python" "conftest.py"))))))
                 (propagated-inputs (list python-numpy python-pystencils python-sympy))
                 (native-inputs
                   (list
                     python-nbformat
                     python-nbconvert
                     python-pytest
                     ))
                 (home-page "https://i10git.cs.fau.de/pycodegen/lbmpy/")
                 (synopsis "Code Generation for Lattice Boltzmann Methods")
                 (description "Code Generation for Lattice Boltzmann Methods")
                 (license #f)))

(concatenate-manifests
  (list
    (specifications->manifest
      (list
        "bash-minimal"
        "grep"
        "which"
        "coreutils"
        "binutils"
        "pkg-config"
        "libtool"

        "openssh"
        "automake"
        "cmake"
        "make"

        "gcc-toolchain@11.3.0"
        "openmpi"
        "openmpi-c++"

        "python-toolchain@3.9.9"
        "python-jinja2"
        "python-py-cpuinfo"
        "pybind11"

        ;; "slurm@21.08.8"

        ;; needs that extra channel guix-hpc, currently I do not know to encode that here in that manifest file
        "likwid"

        ))
    (packages->manifest
      (list python-lbmpy python-pystencils))
    )
  )
