#!/bin/bash -l

L1_CACHE_SIZE=$(getconf -a | grep LEVEL1_DCACHE_SIZE | awk '{print $2}') # BYTE
VECTOR_SIZE=$((L1_CACHE_SIZE / 4000)) # half cache size and in kilo Byte

module load likwid

HOSTNAME=$(hostname)
NUM_CORES=$(nproc --all)
PRECISION="Double"

# look for all available peakflops benchmarks
PERF_BENCHMARKS=$(likwid-bench -a | grep -i "${PRECISION}-precision" | grep "peakflops" | awk -F "-" '{print $1}' | awk '{ printf "%s", $0 } END{ print "" }')

# look for all avaialbe load benchmarks
MEM_BENCHMARKS=$(likwid-bench -a | awk -F "-" '{print $1}' | grep "load" | awk '{ printf "%s", $0 } END{ print "" }')
MEM_BENCHMARKS+=$(likwid-bench -a | awk -F "-" '{print $1}' | grep "copy" | awk '{ printf "%s", $0 } END{ print "" }')
MEM_BENCHMARKS+=$(likwid-bench -a | awk -F "-" '{print $1}' | grep "stream" | awk '{ printf "%s", $0 } END{ print "" }')

current_date=$(date)
mkdir -p ${HOSTNAME}
SUMMARY_FILE="${HOSTNAME}/summary.txt"
rm ${SUMMARY_FILE}

module load python
machinestate -a -o "${HOSTNAME}/machinestate_${HOSTNAME}.json"
likwid-setFrequencies -reset
likwid-setFrequencies -ureset
likwid-setFrequencies -p > "${HOSTNAME}/likwid-setFrequencies.txt"

run_benchmark(){
	benchmarks="$1"
	prefix="$2"
	unit="$3"
	data_size="$4"
	for benchmark in $benchmarks ; do
		out_file="${HOSTNAME}/${prefix}_${HOSTNAME}_${benchmark}.txt"
		cmd="likwid-bench -t $benchmark -W N:${data_size}:$NUM_CORES"
		output=$($cmd) || continue
		{
			echo "$current_date"
			echo "$cmd"
			echo "$output"
		} > $out_file
		result=$(grep "$unit" $out_file | sed 's/\s//g' | awk -v b=$benchmark -F ":" '{printf "%-20s %10s %s", b, $2, $1}')
		echo "$result"
	done
}

{
	echo "$HOSTNAME $current_date:"
	echo "Performance Benchmarks:"
	run_benchmark "$PERF_BENCHMARKS" "perf" "MFlops/s" "$((VECTOR_SIZE*NUM_CORES))kB"
	echo "Bandwidth Benchmarks:"
	run_benchmark "$MEM_BENCHMARKS" "bw" "MByte/s" "2GB"
} | tee -a  ${SUMMARY_FILE}
