import pytest
from unittest.mock import patch
from dashboards.upload import load_config_from_env  # Replace with the correct path to your function



# Test when GRAFANA_API_KEY is missing
@patch('dotenv.load_dotenv')
@patch('os.getenv')
def test_missing_grafana_api_key(mock_getenv, mock_load_dotenv):
    # Setup mock return values with missing GRAFANA_API_KEY
    mock_getenv.side_effect = lambda key: {
        'GRAFANA_SERVER': 'http://test.server'
    }.get(key)

    # Test that ValueError is raised for missing GRAFANA_API_KEY
    with pytest.raises(ValueError, match="GRAFANA_API_KEY is None or not defined in the .env file"):
        load_config_from_env()

# Test when GRAFANA_SERVER is missing
@patch('dotenv.load_dotenv')
@patch('os.getenv')
def test_missing_grafana_server(mock_getenv, mock_load_dotenv):
    # Setup mock return values with missing GRAFANA_SERVER
    mock_getenv.side_effect = lambda key: {
        'GRAFANA_API_KEY': 'test_api_key'
    }.get(key)

    # Test that ValueError is raised for missing GRAFANA_SERVER
    with pytest.raises(ValueError, match="GRAFANA_SERVER is None or not defined in the .env file"):
        load_config_from_env()

# Test when the .env file does not exist
@patch('dotenv.load_dotenv')
def test_no_env_file(mock_load_dotenv):
    # Simulate that the .env file does not exist
    with patch('os.path.exists', return_value=False):
        with pytest.raises(ValueError, match="GRAFANA_API_KEY is None or not defined in the .env file"):
            load_config_from_env()
