from cbutil.gitlab_api import add_pagination, get_last_commit, get_git_infos_from_api, url_encode
import os
import pytest
from unittest import mock


def test_url_encode():
    assert 'walberla%2Fwalberla' == url_encode('walberla/walberla')
    assert 'walberla%2Fwalberla' == url_encode('walberla%2Fwalberla')
    assert 'walberla%2Fwalberla%2Fwalberla' == url_encode('walberla/walberla%2Fwalberla')
    assert 123 == url_encode(123)


def test_pagination():
    base_url = "http://base.url/"
    assert add_pagination(base_url, 1) == f"{base_url}?per_page=20&page=1"
    assert add_pagination(base_url, 1, per_page=1) == f"{base_url}?per_page=1&page=1"


def mock_requests_get_success():
    mock_response = mock.MagicMock()
    mock_response.status_code = 200
    mock_response.json.return_value = [{"id": "123abc", "message": "Test\nCommit"}]
    return mock_response


def mock_requests_get_error():
    mock_response = mock.MagicMock()
    mock_response.status_code = 404
    return mock_response


@pytest.mark.parametrize("project_id", ['ob28imeq%2Fcb-util', 976])
@mock.patch("requests.get")
def test_get_last_commit_success(mock_requests_get, project_id):
    # Define test parameters
    mock_requests_get.return_value = mock_requests_get_success()
    instance_url = 'https://i10git.cs.fau.de/'
    branch = "main"

    # Call the function
    last_commit = get_last_commit(instance_url, project_id, branch)

    # Assertions
    mock_requests_get.assert_called_once_with(
        f"{instance_url}/api/v4/projects/{project_id}/repository/commits?per_page=1&page=1",
        params={"ref_name": branch}
    )
    assert last_commit["id"] == "123abc"
    assert last_commit["message"] == "Test\nCommit"


@pytest.mark.parametrize("project_id", ['ob28imeq%2Fcb-util', 976])
@mock.patch("requests.get")
def test_get_last_commit_api_error(mock_requests_get, project_id):
    # Define test parameters
    mock_requests_get.return_value = mock_requests_get_error()
    instance_url = 'https://i10git.cs.fau.de/'
    branch = "main"
    get_last_commit(instance_url, project_id, branch)
    mock_requests_get.return_value.raise_for_status.assert_called_once()


@pytest.mark.parametrize("project_id", ['ob28imeq%2Fcb-util', 976])
@mock.patch("requests.get")
def test_get_get_git_infos(mock_requests_get, project_id):
    branch_name = 'main'
    instance_url = 'https://i10git.cs.fau.de/'
    mock_requests_get.return_value = mock_requests_get_success()

    git_infos = get_git_infos_from_api(instance_url, project_id)

    assert git_infos['commit'] == "123abc"
    assert git_infos['commit_message'] == "Test Commit"
