import pytest

from dashboards.influx_queries import (
    Query,
    get_variable_condition,
    join_conditions,
    show_tag_values,
)


def test_query():
    q = Query(
        select_="mlupsPerProcess",
        from_="UniformGridGPU",
        where_='"host" =~ /^${host:regex}$/ AND "collisionSetup" =~ /^${collisionSetup:regex}$/ AND $timeFilter ',
        group_by=[
            "blocks_0",
            "blocks_1",
            "blocks_2",
            "cellsPerBlock_0",
            "cellsPerBlock_1",
            "cellsPerBlock_2",
            "gpuBlockSize_0",
            "gpuBlockSize_1",
            "gpuBlockSize_2",
            "collisionSetup",
            "stencil",
            "streamingPattern",
        ],
    )

    q1 = (
        'SELECT "mlupsPerProcess" '
        'FROM "UniformGridGPU" '
        'WHERE ("host" =~ /^${host:regex}$/ AND "collisionSetup" =~ /^${collisionSetup:regex}$/ AND $timeFilter ) '
        'GROUP BY "blocks_0", "blocks_1", "blocks_2", '
        '"cellsPerBlock_0", "cellsPerBlock_1", "cellsPerBlock_2", '
        '"gpuBlockSize_0", "gpuBlockSize_1", "gpuBlockSize_2", '
        '"collisionSetup", "stencil", "streamingPattern"'
    )
    assert q1 == str(q)


def test_show_tag_values():
    s = show_tag_values("UniformGridGPU", "host")
    assert s == 'SHOW TAG VALUES FROM "UniformGridGPU" WITH key = "host"'


def test_get_variable_condtion():
    assert get_variable_condition("host") == '"host" =~ /^${host:regex}$/'
    assert get_variable_condition(" host ") == '"host" =~ /^${host:regex}$/'
    with pytest.raises(ValueError):
        get_variable_condition("")
    with pytest.raises(ValueError):
        get_variable_condition("    ")
    with pytest.raises(ValueError):
        get_variable_condition("\t    ")


def test_join_conditions_two():
    h = get_variable_condition("host")
    c = get_variable_condition("collisionSetup")
    actual = join_conditions([h, c], "AND")
    excpected = '"host" =~ /^${host:regex}$/ AND "collisionSetup" =~ /^${collisionSetup:regex}$/'
    assert actual == excpected


def test_join_conditions_three():
    conds = [
        get_variable_condition(name)
        for name in ["host", "collisionSetup", "cellsPerBlock_0"]
    ]
    # excpected = '"host" =~ /^$host$/ AND "collisionSetup" =~ /^$collisionSetup$/'
    assert join_conditions(conds, "AND") == " AND ".join(conds)
    assert join_conditions(conds, ["AND", "AND"]) == " AND ".join(conds)
    assert join_conditions(conds, ["AND"]) == " AND ".join(conds)
    assert join_conditions(conds, ["OR"]) == " OR ".join(conds)

    excpected = (
        '"host" =~ /^${host:regex}$/ '
        'AND "collisionSetup" =~ /^${collisionSetup:regex}$/ '
        'OR "cellsPerBlock_0" =~ /^${cellsPerBlock_0:regex}$/'
    )
    assert join_conditions(conds, ["AND", "OR"]) == excpected
