import pytest

from dashboards.legends import AxisLabel, Units, build_alias, build_alias_entry


def test_units():
    assert Units.seconds == 's'
    assert Units.milliseconds == 'ms'


def test_labels():
    assert AxisLabel.mlups == 'MLUPS per process'
    assert AxisLabel.runtime == 'Runtime'


@pytest.fixture
def alias_content_single():
    return ("Timestep Strategy", "timeStepStrategy")


def alias_content_multiple(alias_content_single):
    yield alias_content_single
    yield ("Number of Processes", "num_mpi_processes")
    for i in range(3):
        dim = chr(ord('x') + i)
        yield (f"Blocks in {dim} dimension", f"block_{i}")


def test_entry(*alias_content):
    expected = "Timestep Strategy: $tag_timeStepStrategy"
    assert build_alias_entry("Timestep Strategy", "timeStepStrategy") == expected


def test_alias_single(alias_content_single):
    assert build_alias([alias_content_single]) == build_alias_entry(*alias_content_single)


def test_alias_multiple(alias_content_single):
    actual = build_alias(alias_content_multiple(alias_content_single))

    for desc, var_name in alias_content_multiple(alias_content_single):
        assert build_alias_entry(desc, var_name) in actual
