from cbutil.postprocessing import process_linewise
from cbutil.processing_functions import mesa_pd_text, fe2ti_parse_complete_timing, fe2ti_intermediate
from cbutil.util import read_file_line_wise

import pytest


def id(x):
    return x


def test_linewise_trivial():
    lines = ["aaa", "bbb"]
    for raw, processed in zip(lines, process_linewise(id, lines)):
        assert raw == processed


def test_linewise_split():
    def splitter(line):
        splitted = line.split(":")
        return splitted[0], splitted[1]

    lines = ["lhs1:rhs1", "lhs1:rhs1"]

    for raw, processed in zip(lines, process_linewise(splitter, lines)):
        assert splitter(raw) == processed


def test_mesa_pd():
    mesa_pd_example_output = "tests/mesa_pd.txt"
    expected = {'AssocToBlock': 6.43,
                'ContactDetection': 113.55,
                'Euler': 19.65,
                'GenerateLinkedCells': 12.96,
                'ReduceContactHistory': 6.17,
                'ReduceForce': 0.0,
                'SNN': 0.0,
                'SpringDashpot': 60.14,
                'SpringDashpotSpring': 226.38}

    for key, value in process_linewise(mesa_pd_text, read_file_line_wise(mesa_pd_example_output)):
        assert value == expected[key]


TEST_STR_TOTAL = "Absolut_gesamt: 49.09s  Time to solution:  48.95s Create Macro Problem:  0.02s Comm_dup_etc: 0.05s"
TEST_STR_INTERMEDIATE = "load_step 00 lll_pen 00 Newton_load 01 Newton_gesamt 01   micro_solve:  15.61s Comm_for_couple: 0.02s  Time_for_EV: 0.00s  Macro_Solve:  0.56s  Assembly_Macro:  0.55s  KSP_Macro:  0.00s   VecConvert:  0.00s  iter_macro: 01  Micro_HDF5:  0.00s  Comp_W: 0.00s  Ass_Maj: 0.00s  Comm_FLC: 0.00s  FLC_Comp: 0.00s  Ausw_Norm: 0.00s  TimeDest: 0.00  DestBDDC: 0.00  SendBDDC: 0.00  CreateArrBDDC: 0.00 BDDC_Part: 0.00s  PartAss: 0.00s  PartKSP: 0.00s  PartConv: 0.00s  TimeDefo: 0.01s  TimeInt: 0.00s  ||delta_u||: 3.054712e-04"


def test_fe2ti():
    expected = {
        "Absolut_gesamt": 49.09,
        "Time to solution":  48.95,
        "Create Macro Problem":  0.02,
        "Comm_dup_etc": 0.05,

    }
    assert fe2ti_parse_complete_timing(TEST_STR_TOTAL) == expected
    with pytest.raises(ValueError):
        fe2ti_parse_complete_timing(TEST_STR_INTERMEDIATE)


def test_fe2ti_intermedate():
    expected = {
        "step_info": {
            "load_step": "00",
            "lll_pen": "00",
            "Newton_load": "01",
            "Newton_gesamt": "01",
        },
        "fields": {
            "micro_solve":  15.61,
            "Comm_for_couple": 0.02,
            "Time_for_EV": 0.00,
            "Macro_Solve":  0.56,
            "Assembly_Macro":  0.55,
            "KSP_Macro":  0.00,
            "VecConvert":  0.00,
            "iter_macro": 1.0,
            "Micro_HDF5":  0.00,
            "Comp_W": 0.00,
            "Ass_Maj": 0.00,
            "Comm_FLC": 0.00,
            "FLC_Comp": 0.00,
            "Ausw_Norm": 0.00,
            "TimeDest": 0.00,
            "DestBDDC": 0.00,
            "SendBDDC": 0.00,
            "CreateArrBDDC": 0.00,
            "BDDC_Part": 0.00,
            "PartAss": 0.00,
            "PartKSP": 0.00,
            "PartConv": 0.00,
            "TimeDefo": 0.01,
            "TimeInt": 0.00,
            "||delta_u||": 3.054712e-04,
        },
    }
    assert fe2ti_intermediate(TEST_STR_INTERMEDIATE) == expected
    with pytest.raises(ValueError):
        fe2ti_intermediate(TEST_STR_TOTAL)


def test_fe2ti_log():
    test_file = "tests/fe2ti216_bddc_0_1_80_8_1e-8.log"
    assert [*process_linewise(
        lambda line: fe2ti_parse_complete_timing(line.strip()),
        read_file_line_wise(test_file))]
    assert [*process_linewise(
        lambda line: fe2ti_intermediate(line.strip()),
        read_file_line_wise(test_file))]
